; Bloody Human Freeway

; Set screen resolution 640x480
Graphics 640,480

; Enforce double buffering (prevents flickering display)
SetBuffer BackBuffer()

; Set all drawing operations to centre of images
AutoMidHandle True

; Global variables
Global sc, men, man_x, man_y, extra_life, wait, hi

; Create standard gamespeed timer - change value to alter game-speed (lower=slower)
Global frameTimer=CreateTimer(60)

; Global images
Global road, cars, lorrys, human, freeway, life, font, bonus, title, dead, ready, collisionmask

; Global sounds
Global ambient, walk, squish1, ting, background

; Car and truck horns
Dim horn(6)
Dim truck(4)

; Randomise speeds for each lane of traffic
Dim lane_speed(10)

; Randomise traffic
Dim position(11)

; Load multiple images/animated images
cars=animask("gfx\cars.png",32,32,14)
lorrys=animask("gfx\lorrys.png",64,32,8)
human=animask("gfx\man.png",18,24,25)
font=animask("gfx\numbers.png",24,24,10)
bonus=animask("gfx\bonus.png",24,24,6)

; Load single images
road=LoadImage("gfx\road.png")
life=LoadImage("gfx\life.png")
title=LoadImage("gfx\title.png")
dead=LoadImage("gfx\gameover.png")
ready=LoadImage("gfx\ready.png")

MaskImage ready,255,0,255

collisionmask=LoadImage("gfx\collisionmask.png")

MaskImage collisionmask,255,0,255

; Load sounds
horn(0)=LoadSound("sfx\horn1.wav")
horn(1)=LoadSound("sfx\horn2.wav")
horn(2)=LoadSound("sfx\horn3.wav")
horn(3)=LoadSound("sfx\horn4.wav")
horn(4)=LoadSound("sfx\horn5.wav")
horn(5)=LoadSound("sfx\horn6.wav")

truck(0)=LoadSound("sfx\truck1.wav")
truck(1)=LoadSound("sfx\truck2.mp3")
truck(2)=LoadSound("sfx\truck3.wav")
truck(3)=LoadSound("sfx\truck2.mp3")

walk=LoadSound("sfx\walk.wav")
ting=LoadSound("sfx\ting.wav")
ambient=LoadSound("sfx\ambient.wav")
squish1=LoadSound("sfx\squish2.wav")

; Load hi-score
file_in=ReadFile("data\hi.dat")

hi=ReadInt(file_in)

CloseFile(file_in)


; Lorry Type
Type lorry
 Field x,y,dir,image,speed,toot
End Type


; Car Type
Type car
 Field x,y,dir,image,speed,toot
End Type


; Man Type
Type man
 Field x,y,dir,dlay,anim,move,hit
End Type


; Blood Type
Type blood
 Field x,y,dlay
End Type


; Items
Type item
 Field x,y,image,time
End Type

; Activate titlescreen function
title_screen()

End


; Main game function - controls the updating of all game aspects
Function main()

; Randomise traffic speed
timer=MilliSecs()/1000

For j=1 To timer
 For n=0 To 10
  lane_speed(n)=Rand(2,6)
 Next
Next


j=1

; Randomise traffic position
While j<11
 
 n=Rnd(1,10)

 If position(n)=0; And n>0
  position(n)=j
  j=j+1
 EndIf

Wend



; Create the vehicles and man at the start of the game

;          X   value (this value is used for position, speed, direction, image number!)
create_car(224,1)
create_car(160,1)

create_car(192,2)
create_car(320,2)
create_car(448,2)

create_car(416,3)
create_car(480,3)
create_car(544,3)

create_car(128,4)
create_car(320,4)

create_car(192,5)
create_car(256,5)
create_car(512,5)
create_car(576,5)

create_car(352,6)
create_car(416,6)

create_car(448,7)
create_car(512,7)

create_lorry(272,8)
create_lorry(528,8)

create_lorry(80,9)
create_lorry(400,9)

create_lorry(240,10)
create_lorry(336,10)

create_man(320,401)

; Wait 50 frames (display "Ready?") before player can move
wait=50

; Loop background sound
LoopSound ambient

background=PlaySound(ambient)

; Keep playing until alive=false
alive=True

; Main game loop - repeat until all lives depleted
While alive=True

; Create a standard frame rate on all computers - prevents updating until frametimer is ready
WaitTimer(frameTimer)

; Exit game with ESC
If KeyHit(1) Then alive=False

; All lives lost - game over
If men=0 Then alive=False

; Bonus timer - create new random bonus every 5 seconds
bon_time=bon_time+1

; Create bonus item
If bon_time=600

 ; Place random bonus item - X, Y, Image
 create_item(Rnd(12,628),Rnd(64,360),Rnd(0,5))
 
 bon_time=0
EndIf

; Get extra_life
If sc=>extra_life
 men=men+1
 extra_life=extra_life+10000
EndIf

; Draw road
For n=0 To 640 Step 64
 DrawImage road,n,224
Next

; Update hiscore
If sc>hi Then hi=sc

; Draw score text
txt(16,16,sc)

; Draw hi-score txt
txt(300,16,hi)

; Render Types
 ; Blood splat
 For b.blood=Each blood
  render_blood(b)
 Next

 ; Bonus item
 For i.item=Each item
  render_item(i)
 Next

 ; Player hit (so draw before vehicles so appear "under" them)
 For m.man=Each man
  If m\hit=1 Then render_man(m)
 Next

 ; Cars
 For c.car=Each car
  render_car(c)
 Next

 ; Lorrys
 For l.lorry=Each lorry
  render_lorry(l)
 Next
 
 ; Player not hit (so draw after vehicles so appear "on top of" them)
 For m.man=Each man
  If m\hit=0 Then render_man(m)
 Next

; Draw number of lives remaining
For n=1 To men
 DrawImage life,(n*34),446
Next

; Collision detection routine
collision()

; Display "Get Ready" sign
If wait>0
 DrawImage ready,320,240
 wait=wait-1
EndIf

; Flip the screen
Flip False

; Clear screen ready for re-drawing
Cls

; Repeat the main loop
Wend

; Go to GAME OVER function
game_over()

End Function


; Animated image mask function - removes pink borders from images
Function animask(file$,width,height,count)
temphandle=LoadAnimImage(file$,width,height,0,count)

MaskImage temphandle,255,0,255

Return temphandle

End Function


; Create car
Function create_car(x,value)
 c.car=New car
 c\x=x
 c\y=(position(value)*32)+49

 If position(value)<6 
  c\dir=1
 EndIf

 c\image=value-1
 c\speed=lane_speed(value)
End Function


; Create lorry
Function create_lorry(x,value)
 l.lorry=New lorry
 l\x=x
 l\y=(position(value)*32)+49
 
 If position(value)<6 
  l\dir=1
 EndIf

 l\image=value-7
 l\speed=lane_speed(value)
End Function


; Create man
Function create_man(x,y)
 m.man=New man
 m\x=x
 m\y=y
End Function

; Create blood splat
Function create_blood(x,y)
 b.blood=New blood
 b\x=x
 b\y=y
 splat=splat+1
End Function

; Create bonus item
Function create_item(x,y,image)
 i.item=New item
 i\x=x
 i\y=y
 i\image=image
End Function


; Render cars
Function render_car(c.car)

; Move car right
If c\dir=0 Then c\x=c\x+c\speed

; Move car left
If c\dir=1 Then c\x=c\x-c\speed

; Wrap around screen
If c\x<-16 Then c\x=656
If c\x>656 Then c\x=-16

; Toot horn if man too close
distx=Abs(man_x-c\x)
disty=Abs(man_y-c\y)

If disty<16 And distx<64 And c\toot=0
 PlaySound horn(c\image)
 c\toot=100
EndIf

; Prevent multiple tooting
If c\toot>0 Then c\toot=c\toot-1

; Draw car
DrawImage cars,c\x,c\y,(c\image*2)+c\dir

End Function


; Render lorry
Function render_lorry(l.lorry)

; Move lorry right
If l\dir=0 Then l\x=l\x+l\speed

; Move lorry left
If l\dir=1 Then l\x=l\x-l\speed

; Wrap around screen
If l\x<-32 Then l\x=672
If l\x>672 Then l\x=-32

; Toot horn if man too close
distx=Abs(man_x-l\x)
disty=Abs(man_y-l\y)

If disty<16 And distx<96 And l\toot=0
 PlaySound truck(l\image)
 l\toot=100
EndIf

; Prevent multiple tooting
If l\toot>0 Then l\toot=l\toot-1

; Draw lorry
DrawImage lorrys,l\x,l\y,(l\image*2)+l\dir

End Function


; Render man
Function render_man(m.man)

m\move=0

; Move up
If KeyDown(200) And m\hit=0 And wait=0
 m\y=m\y-2
 m\dir=0
 m\dlay=m\dlay+1
 m\move=1
EndIf

; Move down
If KeyDown(208) And m\hit=0 And wait=0
 m\y=m\y+2
 m\dir=2
 m\dlay=m\dlay+1
 m\move=1
EndIf

; Move right
If KeyDown(205) And m\hit=0 And wait=0
 m\x=m\x+2
 m\dir=1
 m\dlay=m\dlay+1
 m\move=1
EndIf

; Move left
If KeyDown(203) And m\hit=0 And wait=0
 m\x=m\x-2
 m\dir=3
 m\dlay=m\dlay+1
 m\move=1
EndIf

; Animate living player
If m\dlay>7
 PlaySound walk
 m\anim=m\anim+1
 m\dlay=0
EndIf

If m\anim>4 And m\hit=0 Then m\anim=0

; Player stops moving
If m\move=0 And m\hit=0
 m\dir=2
 m\anim=0
 m\dlay=0
EndIf

; Player hit by car animation
If m\hit=1
 m\dir=4
 m\dlay=m\dlay+1
EndIf

; Limit player movement
If m\x<8 Then m\x=8
If m\x>632 Then m\x=632
If m\y>401 Then m\y=401

; Reset player at start again
If m\y<48 
 m\x=320
 m\y=401
 sc=sc+1000
 Delay 10
 FlushKeys()
EndIf


man_x=m\x
man_y=m\y

; Draw man
DrawImage human,m\x,m\y,(5*m\dir)+m\anim

; Player died - restart
If m\hit=1 And m\anim=4
 create_blood(m\x,m\y)
 m\dir=2
 m\anim=0
 men=men-1
 m\x=320
 m\y=401
 m\hit=0
 wait=50
EndIf

End Function



; Collision checks
Function collision()

; Man + Car = Splat!
For c.car=Each car
For m.man=Each man
 If ImagesCollide (cars,c\x,c\y,c\image, collisionmask,m\x,m\y,0) And m\hit=0
  PlaySound squish1
  m\hit=1
 EndIf
Next
Next

; Man + Lorry = Splat!
For l.lorry=Each lorry
For m.man=Each man
 If ImagesCollide (lorrys,l\x,l\y,l\image, collisionmask,m\x,m\y,0) And m\hit=0
  PlaySound squish1
  m\hit=1
 EndIf
Next
Next

; Man + bonus = Kerching!
For i.item=Each item
For m.man=Each man
 If ImagesCollide(bonus,i\x,i\y,i\image, collisionmask,m\x,m\y,0) 
  PlaySound ting
  sc=sc+1000
  Delete i
 EndIf
Next
Next

End Function


; Render blood splats
Function render_blood(b.blood)
 DrawImage human,b\x,b\y,24

 ; Blood disappears after a short time
 b\dlay=b\dlay+1

 If b\dlay>120 Then Delete(b)

End Function


; Display bitmap text
Function txt(x,y,ss$)
 xpos=0
  For i=0 To Len(ss$)-1
   cr=(Asc(Mid$(ss$,i+1,1))-48)
	 DrawImage(font,x+(xpos*24),y,cr)
	xpos=xpos+1
  Next
End Function


; Render item
Function render_item(i.item)

; Time countdown
i\time=i\time+1

DrawImage bonus,i\x,i\y,i\image

; Remove item if left for over five seconds
If i\time>300 Then Delete i

End Function


; Title screen
Function title_screen()

quit=0

men=7
sc=0
extra_life=10000

; Destroy any Types that may exist from a previous game
Delete Each car
Delete Each lorry
Delete Each blood
Delete Each item
Delete Each man

; Clear traffic positions
For n=0 To 10
 position(n)=0
Next


; Repeat until starting new game or quitting
While quit=0

 ; Draw titlescreen image
 DrawImage title,320,240

 ; Start game by pressing spacebar
 If KeyHit(57) Then quit=1

 ; Exit game
 If KeyHit(1) Then Exit

 Flip

 Cls

Wend

 ; Start game
 If quit=1 Then main()

End Function


; Game Over
Function game_over()

StopChannel background

time=0

While time<120

DrawImage dead,320,240

time=time+1

; Save high score
 
 file_out=WriteFile("data\hi.dat")

 WriteInt(file_out,hi)

 CloseFile (file_out)

Flip

Cls

Wend

title_screen()

End Function